package se.paddbacken.relayservice;

import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

import com.pi4j.io.gpio.GpioPinDigitalOutput;

@Path("/relays")
@Produces(MediaType.APPLICATION_JSON)
public class RelaysResource {
	private final HashMap<Integer, GpioPinDigitalOutput> pins;

	public RelaysResource(List<GpioPinDigitalOutput> pins) {
		this.pins = new HashMap<Integer, GpioPinDigitalOutput>();
		for (GpioPinDigitalOutput pin : pins) {
			this.pins.put(pin.getPin().getAddress(), pin);
		}
	}
	
	/**
	 * Gets a list of all configured relays including their state.
	 * 
	 * NOTE: The GPIO number is by the scheme used by pi4j a.k.a.
	 * the "generic scheme" where on a 40 pin GPIO header GPIO 0
	 * corresponds to pin 11 etcetera.
	 * 
	 * @return a list of all configured relays.
	 */
	@GET
	public Relay[] getRelays() {
		Relay[] relays = new Relay[pins.size()];
		int index = 0;
		for (Entry<Integer, GpioPinDigitalOutput> entry : pins.entrySet()) {
			relays[index++] = new Relay(entry.getKey(), entry.getValue().isHigh());
		}
		return relays;
	}
	
	/**
	 * Change the state of a relay/GPIO pin.
	 * 
	 * NOTE: The GPIO number is by the scheme used by pi4j a.k.a.
	 * the "generic scheme" where on a 40 pin GPIO header GPIO 0
	 * corresponds to pin 11 etcetera.
	 * 
	 * @param gpio the GPIO number.
	 * @param powered if the GPIO pin/relay is to be powered/on or not.
	 */
	@POST
	@Path("/{gpio}")
	public void powerRelay(@PathParam("gpio") int gpio, @QueryParam("powered") boolean powered) {
		pins.get(gpio).setState(powered);		
	}
}
