package se.paddbacken.relayservice.scheduling;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Path("/lights-scheduler")
@Produces(MediaType.TEXT_PLAIN)
public class LightsSchedulerResource {
	private static final Logger LOGGER = LoggerFactory.getLogger(LightsSchedulerResource.class);
	
	public static class LightEvent {
		private String time;
		private final boolean lightsOn;

		public LightEvent(String time, boolean lightsOn) {
			this.time = time;
			this.lightsOn = lightsOn;
		}

		public String getTime() {
			return time;
		}

		public void setTime(String time) {
			this.time = time;
		}

		public boolean isLightsOn() {
			return lightsOn;
		}
	}
	
	public static interface Listener {
		void handleLightsEvent(boolean switchLightsOn);
	}
	
	private boolean eventsEnabled;
	private final List<LightEvent> lightEvents;
	
	public LightsSchedulerResource() {
		eventsEnabled = true;
		lightEvents = Collections.unmodifiableList(Arrays.asList(
				new LightEvent("00:00", false), new LightEvent("06:00", true),
				new LightEvent("08:00", false), new LightEvent("17:00", true)));
	}
	
	public void start(Listener listener) {
		Thread thread = new Thread() {
			@Override
			public void run() {
				String lastSystemTime = "";
				while (true) {
					try {
						String systemTime = getSystemTime();
						if (!systemTime.equals(lastSystemTime)) {							
							lastSystemTime = systemTime;
							for (LightEvent lightEvent : lightEvents) {
								if (eventsEnabled && lightEvent.getTime().equals(systemTime)) {
									listener.handleLightsEvent(lightEvent.isLightsOn());
									break;
								}
							}
						}
						Thread.sleep(200L);
					} catch (Exception e) {
						LOGGER.error("Error in scheduler thread: " + e, e);
					}
				}
			}
		};
		thread.setDaemon(true);
		thread.start();
	}


	@GET
	@Path("/events-enabled")
	public boolean isEventsEnabled() {
		return eventsEnabled;
	}

	@POST
	@Path("/")
	public void setEventsEnabled(@QueryParam("eventsEnabled")boolean eventsEnabled) {
		this.eventsEnabled = eventsEnabled;
	}
	
	public String getSystemTime() {
		return new SimpleDateFormat("HH:mm").format(new Date());
	}

	@GET
	@Path("/light-events")
	@Produces(MediaType.APPLICATION_JSON)
	public List<LightEvent> getLightEvents() {
		return lightEvents;
	}
}
