package se.paddbacken.relayservice.ui;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.apache.commons.io.IOUtils;

@Path("/ui")
@Produces(MediaType.TEXT_HTML)
public class RelayServiceUIResource {
	private static final Map<String, String> CONTENT_TYPES_BY_RESOURCE_SUFFIX;
	private static final Map<String, Object> CACHED_RESOURCES = new HashMap<String,Object>();
	
	static {
		HashMap<String, String> contentTypesByFileSuffix = new HashMap<String,String>();
		contentTypesByFileSuffix.put("html", MediaType.TEXT_HTML);
		contentTypesByFileSuffix.put("css", "text/css");
		contentTypesByFileSuffix.put("js", "application/javascript");
		contentTypesByFileSuffix.put("jpg", "image/jpeg");
		contentTypesByFileSuffix.put("png", "image/png");
		CONTENT_TYPES_BY_RESOURCE_SUFFIX = Collections.unmodifiableMap(contentTypesByFileSuffix);
	}
	
	private final SysInfoResource sysInfoResource;
	
	public RelayServiceUIResource(SysInfoResource sysInfoResource) {
		this.sysInfoResource = sysInfoResource;		
	}
	
	@GET
	public Response getDefault() throws URISyntaxException {
		return Response.temporaryRedirect(new URI("ui/relays.html")).build();
	}
	
	@GET
	@Path("{resourceName}")
	public Response getUiResource(@PathParam("resourceName") String resourceName)
			throws URISyntaxException, IOException {
		if (resourceName.endsWith("sysinfo.html")) {
			return Response.ok(sysInfoResource.getSysInfo(), MediaType.TEXT_HTML).build();			
		}
		
		String mediaType = CONTENT_TYPES_BY_RESOURCE_SUFFIX.get(
				resourceName.toLowerCase().substring(resourceName.lastIndexOf('.') + 1));		
		synchronized (CACHED_RESOURCES) {
			Object entity = CACHED_RESOURCES.get(resourceName);
			if (entity == null) {
				InputStream inputStream = getClass().getResourceAsStream("/ui/" + resourceName);
				if (inputStream == null) {
					return Response.status(404).build();
				} else {
					entity = (mediaType.startsWith("text") || mediaType.contains("javascript"))
							? IOUtils.toString(inputStream)
							: IOUtils.toByteArray(inputStream);
					CACHED_RESOURCES.put(resourceName, entity);
				}
			}
			return Response.ok(entity, mediaType).build();
		}
	}
}
