package se.paddbacken.relayservice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.Comparator;

import org.junit.Before;
import org.junit.Test;

import com.pi4j.io.gpio.GpioPinDigitalOutput;
import com.pi4j.io.gpio.Pin;

public class RelaysResourceTest {
	private static final int address0 = 78634;
	private static final int address1 = 489534;
	
	private RelaysResource relaysResource;
	private GpioPinDigitalOutput output1;

	@Before
	public void setup() {
		output1 = createGpioPinDigitalOutput(address1, false);
		relaysResource = new RelaysResource(Arrays.asList(
				createGpioPinDigitalOutput(address0, true),
				output1));
	}
	
	private GpioPinDigitalOutput createGpioPinDigitalOutput(int address, boolean high) {
		GpioPinDigitalOutput output = mock(GpioPinDigitalOutput.class);
		Pin pin = mock(Pin.class);
		when(output.getPin()).thenReturn(pin);
		when(pin.getAddress()).thenReturn(address);
		when(output.isHigh()).thenReturn(high);
		return output;
	}
	
	@Test
	public void shouldReadRelayStatesAndReturnAsJSonObjectsWhenGettingRelays() {
		Relay[] relays = relaysResource.getRelays();
		assertNotNull(relays);
		assertEquals(2, relays.length);
		Arrays.sort(relays, new Comparator<Relay>() {
			@Override
			public int compare(Relay o1, Relay o2) {
				return Integer.valueOf(o1.getGpio()).compareTo(Integer.valueOf(o2.getGpio()));
			}
		});
		assertEquals(address0, relays[0].getGpio());
		assertTrue(relays[0].isPowered());
		assertEquals(address1, relays[1].getGpio());
		assertFalse(relays[1].isPowered());
	}
	
	@Test
	public void shouldSetRelayPowerOfSelectedGpioWhenSettingRelayPower() {
		relaysResource.powerRelay(address1, false);
		verify(output1).setState(false);		
	}
}
